#ifndef __vect3t_h
#define __vect3t_h

#include <iostream>
#include <cmath>

template<class FP> class vect3 {
public:
  vect3() : x1(0), x2(0), x3(0)
  {
  }
  vect3(FP _x1, FP _x2, FP _x3)
    : x1(_x1), x2(_x2), x3(_x3)
  {
  }
  vect3(const vect3<FP>& v) : x1(v.x1), x2(v.x2), x3(v.x3)
  {
  }

  /* naten */
  friend std::istream& operator>> (std::istream &in, vect3<FP>& v)
  {
    return in >> v.x1 >> v.x2 >> v.x3;
  }
  /* vpis */
  friend std::ostream& operator<< (std::ostream &out, const vect3<FP>& v)
  {
    return out << v.x1 << " " << v.x2 << " " << v.x3;
  }
  /* souet */
  friend vect3<FP> operator+ (const vect3<FP>& v1, const vect3<FP>& v2)
  {
    return vect3(v1.x1 + v2.x1, v1.x2 + v2.x2, v1.x3 + v2.x3);
  }
  /* rozdl */
  friend vect3<FP> operator- (const vect3<FP>& v1, const vect3<FP>& v2)
  {
    return vect3(v1.x1 - v2.x1, v1.x2 - v2.x2, v1.x3 - v2.x3);
  }
  /* opan vektor */
  friend vect3<FP> operator- (const vect3<FP>& v)
  {
    return vect3(-v.x1, -v.x2, -v.x3);
  }
  /* nsoben skalrem */
  friend vect3<FP> operator* (const vect3<FP>& v, const FP fp)
  {
    return vect3(v.x1 * fp, v.x2 * fp, v.x3 * fp);
  }
  friend vect3<FP> operator* (const FP fp, const vect3<FP>& v)
  {
    return v * fp;
  }
  /* ortonormln skalrn souin */
  friend FP operator* (const vect3<FP>& v1, const vect3<FP>& v2)
  {
    return v1.x1 * v2.x1 + v1.x2 * v2.x2 + v1.x3 * v2.x3;
  }
  /* vektorov souin */
  friend vect3<FP> operator% (const vect3<FP>& v1, const vect3<FP>& v2)
  {
    return vect3(v1.x2 * v2.x3 - v1.x3 * v2.x2,
	         v1.x3 * v2.x1 - v1.x1 * v2.x3,
		 v1.x1 * v2.x2 - v1.x2 * v2.x1);
  }

  /* dlka vektoru */
  FP abs() const
  {
    return sqrt(x1 * x1 + x2 * x2 + x3 * x3);  
  }
  friend FP abs(const vect3<FP>& v)
  {
    return v.abs();
  }
  /* smen souin */
  friend FP m_product(const vect3<FP>& v1, const vect3<FP>& v2,
    const vect3<FP>& v3)
  {
    return v1 * (v2 % v3);
  }
  
private:
  FP x1, x2, x3;
};

#endif /* __vect3t_h */
