#include <iostream>
#include <cmath>
#include <cstring>
#include "tvary.h"

/****************************************************************************
 * tda: chybakresleni                                                     *
 * el : pedvn chybovch stav, informace o jejich vznamu             *
 *        je kompatibiln se standardn tdou std::exception               *
 ****************************************************************************/

char *chybakresleni::chyby[] = {
  "jin chyba",
  "neplatn rozmry",
  "mlo pamti",
  "neplatn souadnice"
};

/* tda pro pedvn chyb */
chybakresleni::chybakresleni(chyba _c)
  : c(_c)
{
}

chybakresleni::~chybakresleni() throw()
{
}

const char *chybakresleni::what() const throw()
{
  if ((c >= JINA_CHYBA) && (c < POCET_CHYB))
    return chyby[c];
  else
    return chyby[JINA_CHYBA];
}

/****************************************************************************
 * tda: plocha                                                            *
 * el : obrzek v pamti, zkladn operace s nm - get/putpixel, vstup   *
 ****************************************************************************/

/* zkladn konstruktor */
plocha::plocha() throw(chybakresleni)
{
  plocha_(79, 23, '*', ' ', false);
}

/* konstruktor pro uivatelem danou velikost */
plocha::plocha(int _vel_x, int _vel_y) throw(chybakresleni) 
{
  plocha_(_vel_x, _vel_y, '*', ' ', false);
}

/* konstruktor pro uivatelem dan "barvy pixel" */
plocha::plocha(char _pixel, char _pozadi) throw(chybakresleni) 
{
  plocha_(79, 23, _pixel, _pozadi, false);
}

/* speciln konstruktor */
plocha::plocha(int _vel_x, int _vel_y,
  char _pixel, char _pozadi, bool _prisna_kontrola) throw(chybakresleni) 
{
  plocha_(_vel_x, _vel_y, _pixel, _pozadi, _prisna_kontrola);
}

/* koprovac konstruktor */
plocha::plocha(plocha &p) throw(chybakresleni)
{
  plocha_(p.velikost_x(), p.velikost_y(), p.pixel, p.pozadi,
    p.prisna_kontrola);
  memcpy(data, p.data, sizeof(int) * p.velikost_x() * p.velikost_y());
}

/* vkonn sloka konstruktoru */
void plocha::plocha_(int _vel_x, int _vel_y,
  char _pixel, char _pozadi,
  bool _prisna_kontrola) throw(chybakresleni) 
{
  if ((_vel_x <= 0) || (_vel_y <= 0))
    throw chybakresleni(chybakresleni::NEPLATNE_ROZMERY);
  if ((data = new int[_vel_x * _vel_y]) == 0)
    throw chybakresleni(chybakresleni::MALO_PAMETI);

  vel_x = _vel_x;
  vel_y = _vel_y;
  pixel = _pixel;    
  pozadi = _pozadi;
  prisna_kontrola = _prisna_kontrola;

  vyprazdni();
}

/* destruktor */
plocha::~plocha()
{
  delete[] data;
}

/* smazn plochy */
void plocha::vyprazdni(int vypln) throw()
{
  for (int i = 0; i < vel_x * vel_y; i++)
    data[i] = vypln;
}

/* getpixel/putpixel :-) */
int plocha::getpixel(int x, int y) const throw(chybakresleni)
{
  if ((x < 0) || (y < 0) || (x >= vel_x) || (y >= vel_y))
    if (prisna_kontrola)
      throw chybakresleni(chybakresleni::NEPLATNE_SOURADNICE);
    else
      return 0;
  return data[y * vel_x + x];
}

void plocha::putpixel(int x, int y, int pixel) throw(chybakresleni)
{
  if ((x < 0) || (y < 0) || (x >= vel_x) || (y >= vel_y))
    if (prisna_kontrola)
      throw chybakresleni(chybakresleni::NEPLATNE_SOURADNICE);
    else
      return;
  data[y * vel_x + x] += pixel; 
}

/* zobrazen plochy do proudu */
std::ostream& operator<<(std::ostream &vystup, plocha &p)
{
  /* speciln reim kompatibility se vzorovm programem :-) */
  for (int j = p.velikost_y() - 1; j >= 0; j--)
  {
    for (int i = 0; i < p.velikost_x(); i++)
      vystup << ((p.data[j * p.velikost_x() + i] > 0) ? p.pixel : p.pozadi);
    vystup << std::endl;
  }
  return vystup;
}

#ifdef ROTATE
/* totko */
void plocha::otoc(int x0, int y0, double a,
  int zvyrazneni) throw(chybakresleni)
{
  double _x, _y;
  int x, y;
  /* kopie */
  plocha stara_plocha(*this);
  /* vyistme plochu */
  vyprazdni(zvyrazneni);
 
  a = a * M_PI / 180;
  /* NKOLIK DK LINERN ALGEBRY...
   * Otome plochu proti smru hodinovch ruiek tak, e si nejprve
   * posuneme sted souadn soustavy do stedu oten. Potom kad bod
   * zobrazme podle vztahu
   *
   * / x \   /  cos a  -sin a \ / _x \
   * |   | = |                | |    |
   * \ y /   \  sin a   cos a / \ _y /
   *
   * Nakonec pesuneme sted souadnho systmu do pvodnho msta.
   */
  for (int j = 0; j < vel_y; j++)
    for (int i = 0; i < vel_x; i++)
    {
      x = i;
      y = j;
      _x = x - x0;
      _y = y - y0;
      x = static_cast<int>(rint(cos(a) * _x - sin(a) * _y) + x0);
      y = static_cast<int>(rint(sin(a) * _x + cos(a) * _y) + y0);
      if ((x >= 0) && (x < vel_x) && (y >= 0) && (y < vel_y))
        putpixel(i, j, stara_plocha.getpixel(x, y));
    }
}
#endif

/****************************************************************************
 * tda: bod                                                               *
 * el : vykreslovn jednotlivch bod na plochu                          *
 ****************************************************************************/

/* konstruktor */
bod::bod() throw()
  : _kolikrat(0)
{
}

bod::bod(int _x, int _y) throw()
  : x(_x), y(_y)
{
}

/* destruktor */
bod::~bod()
{
}

/* nakresl bod */
void bod::vykresli(plocha &p, int c) throw(chybakresleni)
{
  p.putpixel(x, y, c);
  pridej();
}

/* nakresl bod bez barvy = smae jej */
void bod::schovej(plocha &p) throw(chybakresleni)
{
  if (uber())
    vykresli(p, -1);
}

/****************************************************************************
 * tda: cara                                                              *
 * el : vykreslovn ar na plochu                                        *
 ****************************************************************************/

/* konstruktor */
cara::cara(int _x1, int _y1, int _x2, int _y2) throw()
  : x1(_x1), y1(_y1), x2(_x2), y2(_y2)
{
}

/* destruktor */
cara::~cara()
{
}

/* nakresl ru Bresenhamovm algoritmem */
void cara::vykresli(plocha &p, int c) throw(chybakresleni)
{
  int d, dx, dy, ai, bi, xi, yi;

  pridej();

  if ( (x1 == x2) && (y1 == y2) )
  {
    p.putpixel(x1, y1, c);
    return;
  }

  int _x1 = x1, _y1 = y1, _x2 = x2, _y2 = y2;
  
  if (_x1 < _x2)
  {
    xi = 1;
    dx = _x2 - _x1;
  }
  else
  {
    xi = -1;
    dx = _x1 - _x2;
  }

  if (_y1 < _y2)
  {
    yi = 1;
    dy = _y2 - _y1;
  }
  else
  {
    yi = -1;
    dy = _y1 - _y2;
  }

  p.putpixel(_x1, _y1, c);

  if (dx > dy)
  {
    ai = (dy - dx) * 2;
    bi = dy * 2;
    d  = bi - dx;

    do {
      if (d >= 0)
      {
        _y1 += yi;
        d  += ai;
      }
      else
        d  += bi;

      _x1 += xi;
      p.putpixel(_x1, _y1, c);
    } while (_x1 != _x2);
  }
  else
  {
    ai = (dx - dy) * 2;
    bi = dx * 2;
    d  = bi - dy;

    do {
      if (d >= 0)
      {
        _x1 += xi;
        d  += ai;
      }
      else
        d  += bi;

      _y1 += yi;
      p.putpixel(_x1, _y1, c);
    } while (_y1 != _y2);
  }
}

/****************************************************************************
 * tda: ctverec                                                           *
 * el : vykreslovn tverc na plochu                                    *
 ****************************************************************************/

/* konstruktor */
ctverec::ctverec(int _x, int _y, int _a) throw()
  : x(_x), y(_y), a(_a >= 0 ? _a : -_a)
{
}

/* destruktor */
ctverec::~ctverec()
{
}

/* vykreslen tverce */
void ctverec::vykresli(plocha &p, int c) throw(chybakresleni)
{
  int i;
  
  pridej();
  
  for (i = x; i < x + a; i++)
  {
    p.putpixel(i, y, c);
    p.putpixel(i, y + a - 1, c);
  }
  for (i = y + 1; i < y + a - 1; i++)
  {
    p.putpixel(x, i, c);
    p.putpixel(x + a - 1, i, c);
  }
}

/****************************************************************************
 * tda: kruznice                                                          *
 * el : vykreslovn krunic (a elips) na plochu                          *
 ****************************************************************************/

/* konstruktor */
kruznice::kruznice(int _mx, int _my, int _a) throw()
  : mx(_mx), my(_my), a(_a >= 0 ? _a : -_a), b(a)
{
}

kruznice::kruznice(int _mx, int _my, int _a, int _b) throw()
  : mx(_mx), my(_my), a(_a >= 0 ? _a : -_a), b(_b >= 0 ? _b : -_b)
{
}

/* destruktor */
kruznice::~kruznice()
{
}

/* vykreslen krunice */
void kruznice::vykresli(plocha &p, int c) throw(chybakresleni)
{
  int x, mx1, mx2, my1, my2;
  long int aq, bq, dx, dy, r, rx, ry;

  pridej();

  p.putpixel(mx + a, my, c);
  p.putpixel(mx - a, my, c);

  mx1 = mx - a;   my1 = my;
  mx2 = mx + a;   my2 = my;

  aq = a * a;
  bq = b * b;
  dx = aq << 1;
  dy = bq << 1;
  r  = a * bq;
  rx = r << 1;
  ry = 0;
  x = a;

  while (x > 0)
  {
    if (r > 0)
    {
      my1++;
      my2--;
      ry += dx;
      r  -= ry;
    }
    else
    {
      x--;
      mx1++;
      mx2--;
      rx -= dy;
      r  += rx;
    }
    p.putpixel(mx1, my1, c);
    p.putpixel(mx1, my2, c);
    p.putpixel(mx2, my1, c);
    p.putpixel(mx2, my2, c);
  }
}
