#ifndef __tvary_h
#define __tvary_h

#include <iostream>
#include <exception>

/****************************************************************************
 * tda: chybakresleni                                                     *
 * el : pedvn chybovch stav, informace o jejich vznamu             *
 *        je kompatibiln se standardn tdou std::exception               *
 ****************************************************************************/

class chybakresleni : public std::exception {
public:
  enum chyba {
    JINA_CHYBA=0,
    NEPLATNE_ROZMERY,
    MALO_PAMETI,
    NEPLATNE_SOURADNICE,
    POCET_CHYB
  };

  chybakresleni(chyba _c = JINA_CHYBA);
  virtual ~chybakresleni() throw();
  virtual const char *what() const throw();
  int ID() const { return c; }
private:
  chyba c;
  static char *chyby[];
};

/****************************************************************************
 * tda: plocha                                                            *
 * el : obrzek v pamti, zkladn operace s nm - get/putpixel, vstup   *
 ****************************************************************************/

class plocha {
public:
  /* univerzln konstruktory pro pohodlnj pouit */
  plocha() throw(chybakresleni);
  plocha(char _pixel, char _pozadi) throw(chybakresleni);
  plocha(int _vel_x, int _vel_y) throw(chybakresleni);
  plocha(int _vel_x, int _vel_y,
    char _pixel, char _pozadi,
    bool _prisna_kontrola = false) throw(chybakresleni);  
  /* koprovac konstruktor */
  plocha(plocha &p) throw(chybakresleni);
 /* destruktor */ 
  ~plocha();
  
  /* smazn plochy */
  void vyprazdni(int vypln = 0) throw();
  /* getpixel/putpixel :-) */
  int getpixel(int x, int y) const throw(chybakresleni);
  void putpixel(int x, int y, int pixel) throw(chybakresleni);
  /* zobrazen plochy do proudu */
  friend std::ostream& operator<<(std::ostream &vystup, plocha &p);

#ifdef ROTATE  
  /* totko s volitelnm zvraznnm hranic otoen plochy pomoc
   * vyplnnho pozad */
  void otoc(int x, int y, double a, int zvyrazneni = 0) throw(chybakresleni);
#endif  

  int velikost_x() const throw() { return vel_x; }
  int velikost_y() const throw() { return vel_y; }
  
private:
  int vel_x, vel_y;
  char pixel, pozadi;
  bool prisna_kontrola;
  int *data;

  /* vlastn konstroktor */
  void plocha_(int _vel_x, int _vel_y,
    char _pixel, char _pozadi, bool _prisna_kontrola) throw(chybakresleni);
 };

/****************************************************************************
 * tda: bod                                                               *
 * el : vykreslovn jednotlivch bod na plochu                          *
 ****************************************************************************/

class bod {
public:
  bod(int _x, int _y) throw();
  virtual ~bod();
  virtual void vykresli(plocha &p, int c = 1) throw(chybakresleni);
  virtual void schovej(plocha &p) throw(chybakresleni);
protected:
  bod() throw();
  void pridej() { _kolikrat++; }
  bool uber() { if (_kolikrat>0) { _kolikrat--; return true; } return false; }
  int kolikrat() { return _kolikrat; }
private:
  int x, y;
  int _kolikrat;
};

/****************************************************************************
 * tda: cara                                                              *
 * el : vykreslovn ar na plochu                                        *
 ****************************************************************************/

class cara : public bod {
public:
  cara(int _x1, int _y1, int _x2, int _y2) throw();
  virtual ~cara();
  virtual void vykresli(plocha &p, int c = 1) throw(chybakresleni);
private:
  int x1, y1, x2, y2;
};

/****************************************************************************
 * tda: ctverec                                                           *
 * el : vykreslovn tverc na plochu                                    *
 ****************************************************************************/

class ctverec : public bod {
public:
  ctverec(int _x, int _y, int _a) throw();
  virtual ~ctverec();
  virtual void vykresli(plocha &p, int c = 1) throw(chybakresleni);
private:
  int x, y, a;
};

/****************************************************************************
 * tda: kruznice                                                          *
 * el : vykreslovn krunic (a elips) na plochu                          *
 ****************************************************************************/

class kruznice : public bod {
public:
  kruznice(int _mx, int _my, int _a) throw();
  kruznice(int _mx, int _my, int _a, int _b) throw ();
  virtual ~kruznice();
  virtual void vykresli(plocha &p, int c = 1) throw (chybakresleni);
private:
  int mx, my, a, b;
};
 
#endif
